@@ -38,6 +38,7 @@
 #' @param epsilon Defaults to 1e-8. Positive convergence tolerance \eqn{\epsilon} that is directly passed to the \code{control} argument of \code{mclogit::mblogit} as \code{mclogit.control}. Only for \code{method='mle', group.var=...}.
 #' @param ncores The number of cores to parallelize to, see \sQuote{Details}. If >0, the number of CPU cores to be used. -1 for all available -1 core. Only for \code{method="mle"}.
 #' @param cluster.type The type of cluster to be used, see \code{?parallel::makeCluster}. \code{abn} then defaults to \code{"PSOCK"} on Windows and \code{"FORK"} on Unix-like systems. With "FORK" the child process are started with \code{rscript_args = "--no-environ"} to avoid loading the whole workspace into each child.
+#' @param only_glmmTMB_poisson logical, if TRUE only use \code{glmmTMB} to fit Poisson nodes with random effects. This is useful if \code{glmer} fails due to convergence issues. Default is FALSE.
 #' @param seed a non-negative integer which sets the seed in \code{set.seed(seed)}.
 #'
 #' @details
@@ -64,6 +65,7 @@
 #'                         trace.mblogit = FALSE,
 #'                         catcov.mblogit = "free",
 #'                         epsilon = 1e-6,
+#'                         only_glmmTMB_poisson=FALSE,
 #'                         seed = 9062019L)
 #' ctrlbayes <- abn::build.control(method = "bayes",
 #'                            max.mode.error = 10,
@@ -121,6 +123,7 @@ build.control <-
             trace.mblogit = FALSE,
             catcov.mblogit = "free",
             epsilon = 1e-6,
+            only_glmmTMB_poisson=FALSE,
             seed = 9062019L) {
     if (method == "bayes") {
       ctrl <-
@@ -167,6 +170,7 @@ build.control <-
           trace.mblogit = trace.mblogit,
           catcov.mblogit = catcov.mblogit,
           epsilon = epsilon,
+          only_glmmTMB_poisson=only_glmmTMB_poisson,
           seed = seed
         )
     } else {
